#include<cstdlib>
#include<iostream>
#include<vector>
#include<algorithm>

#include<ctime>
#include<random>

typedef unsigned short ushort;

bool isDivisibleByFour ( const ushort &Arg ) {
  return ( ( ( Arg % 4 ) == 0 ) );
}

bool veryExpensiveComparison ( const ushort &A, const ushort &B ) {
  std::cout << "VERY EXPENSIVE COMPARISON" << std::endl;
  return ( ( A < B ) );
}

template<typename ContainerType>
void printContainer ( const ContainerType &Arg ) {
  for ( auto it = Arg.begin(); it != Arg.end(); ++it ) {
    std::cout << *it << " ";
  }
  std::cout << std::endl;
}



int main ( const int, const char** const ) {
  const size_t mySize = 10;

  // std::find works with C arrays as well as STL data structures

  const ushort myArray[mySize] = { 0, 2, 4, 6, 8, 10, 12, 14, 12, 12 };
  const ushort* p = std::find ( myArray, myArray+mySize, 12 );
  if ( ( p - myArray ) < mySize ) {
    std::cout << "Found " << *p << " at position " << p - myArray << std::endl;
  } // else, p == myArray+mySize indicating not found

  std::vector<ushort> myVector ( myArray, myArray+mySize );
  std::vector<ushort>::iterator pos = std::find ( myVector.begin(), myVector.end(), 14 );
  if ( pos != myVector.end() ) {
    std::cout << "Found " << *pos << " at position " << pos - myVector.begin() << std::endl;
  } // else not found


  // same for std::count

  const int numCount = std::count ( myArray, myArray+mySize, 12 );
  std::cout << "12 is contained " << numCount << " times." << std::endl;
  std::cout << std::endl;


  // std::shuffle in combination with PRNG

  printContainer ( myVector );
  unsigned int mySeed = static_cast<int> ( std::time(0) );
  std::cout << "Shuffling with seed " << mySeed << std::endl;
  std::shuffle ( myVector.begin(), myVector.end(), std::mt19937 ( mySeed ) );
  printContainer ( myVector );
  std::cout << std::endl;


  // std::minmax_element

  std::pair< std::vector<ushort>::iterator, std::vector<ushort>::iterator >
    minMax = std::minmax_element ( myVector.begin(), myVector.end() );

  std::cout << "Minimum of " << * ( minMax.first  ) <<  " at " << minMax.first  - myVector.begin() << ", "
            << "maximum of " << * ( minMax.second ) <<  " at " << minMax.second - myVector.begin() << std::endl << std::endl;


  // std::partition

  std::vector<ushort> myVector2 ( myVector );
  std::cout << "Partitioning (non-stable)" << std::endl;
  std::partition ( myVector2.begin(), myVector2.end(), isDivisibleByFour );
  printContainer ( myVector2 );
  std::cout << std::endl;

  // std::stable_partition

  std::cout << "Partitioning (stable)" << std::endl;
  std::stable_partition ( myVector.begin(), myVector.end(), isDivisibleByFour );
  printContainer ( myVector );
  std::cout << std::endl;

  // std::binary_search

  // we can't really appreciate the benefit of binary search here as equality is expressed by comparison
  // just imagine a much larger vector where also comparison for equality is expensive
  bool found = std::binary_search ( myVector.begin(), myVector.end(), 14, veryExpensiveComparison );

  return ( EXIT_SUCCESS );
}
