#include<cstdlib>
#include<iostream>
#include<vector>
#include<set>
#include<algorithm>
#include<functional>

template<typename FloatDataType>
struct lessAsInt {
  bool operator() ( const FloatDataType A, const FloatDataType B ) const {
    return ( ( static_cast<int> ( A ) < static_cast<int> ( B ) ) );
  }
};


template<typename ContainerType>
void printContainer ( const ContainerType &Arg ) {
  for ( auto it = Arg.begin(); it != Arg.end(); ++it ) {
    std::cout << *it << " ";
  }
  std::cout << std::endl;
}



int main ( const int, const char** const ) {

  const size_t mySize = 8;
  float myArray[mySize] = { 0.2, 0.8, 1.9, 1.7, 0.3, 1.1, 1.5, 0.4 };

  {
    std::cout << "Original vector" << std::endl;
    std::vector<float> myVector ( myArray, myArray+mySize );
    printContainer ( myVector );
    std::cout << std::endl;
  }

  {
    std::cout << "standard sort" << std::endl;
    std::vector<float> myVector ( myArray, myArray+mySize );
    std::sort ( myVector.begin(), myVector.end() );
    printContainer ( myVector );
    std::cout << std::endl;
  }

  {
    std::cout << "standard backwards sort" << std::endl;
    std::vector<float> myVector ( myArray, myArray+mySize );
    std::sort ( myVector.begin(), myVector.end(), std::greater<float>() );
    printContainer ( myVector );
    std::cout << std::endl;
  }

  {
    std::cout << "set with backwards ordering" << std::endl;
    std::set<float, std::greater<float> > mySet ( myArray, myArray+mySize );
    printContainer ( mySet );
    std::cout << std::endl;
  }

  {
    std::cout << "sort by magnitude" << std::endl;
    std::vector<float> myVector ( myArray, myArray+mySize );
    std::sort ( myVector.begin(), myVector.end(), lessAsInt<float>() );
    printContainer ( myVector );
    std::cout << std::endl;
  }

  {
    std::cout << "Sort C array" << std::endl;
    std::sort ( myArray, myArray+mySize );
    std::vector<float> myVector ( myArray, myArray+mySize );
    printContainer ( myVector );
    std::cout << std::endl;
  }

  return ( EXIT_SUCCESS );
}
