#include<cstdlib>
#include<iostream>
#include<list>
#include<forward_list>
#include<ctime>
#include<chrono>

template<typename ListType>
void fillWithSomeData ( ListType &Arg ) {
  const unsigned int destSize = 10;
  for ( unsigned int i = 0; i < destSize; ++i ) {
    Arg.push_front ( 42*i+23 );
  }
}

void benchmarkList ( const size_t ListSize ) {
  std::list<signed long int> myListC;
  fillWithSomeData ( myListC );
  std::cout << "Modifying list … " << std::flush;
  std::chrono::high_resolution_clock::time_point t1 = std::chrono::high_resolution_clock::now();
  for ( size_t i = 0; i < ListSize; ++i ) {
    std::list<signed long int>::iterator it = myListC.begin();
    ++it;
    myListC.insert ( it, 4711 );
  }
  std::chrono::high_resolution_clock::time_point t2 = std::chrono::high_resolution_clock::now();
  const std::chrono::duration<double> timeSpan = std::chrono::duration_cast<std::chrono::duration<double>>(t2 - t1);
  std::cout << "done, took " << timeSpan.count() << std::endl;
}


void benchmarkForwardList ( const size_t ListSize ) {
  std::forward_list<signed long int> myListD;
  fillWithSomeData ( myListD );
  std::cout << "Modifying forward list … " << std::flush;
  std::chrono::high_resolution_clock::time_point t1 = std::chrono::high_resolution_clock::now();
  for ( size_t i = 0; i < ListSize; ++i ) {
    std::forward_list<signed long int>::iterator it = myListD.begin();
    ++it;
    myListD.insert_after ( it, 4711 );
  }
  std::chrono::high_resolution_clock::time_point t2 = std::chrono::high_resolution_clock::now();
  const std::chrono::duration<double> timeSpan = std::chrono::duration_cast<std::chrono::duration<double>>(t2 - t1);
  std::cout << "done, took " << timeSpan.count() << std::endl;
}


int main ( const int, const char** const ) {
  const size_t listSize = 32*1024*1024;

  benchmarkList ( listSize );

  benchmarkForwardList ( listSize );

  return ( EXIT_SUCCESS );
}
