#include<cstdlib>
#include<iostream>
#include<map>
#include<unordered_map>
#include<string>

int main ( const int, const char** const ) {
  {
    std::map<short int, std::string> myMap;
    myMap[1] = "One"; // inserting elements works via operator[]
    myMap[2] = "Two";
    std::string dwarf = myMap[3];

    // access including existence check and const access via at()
    for ( short int i = 0; i < 5; ++i ) {
      std::string myString;
      try {
        myString = myMap.at ( i );
      } catch ( std::exception &ex ) {
        std::cout << "exception caught: " << ex.what() << std::endl;
        myString = "not found";
      }
      std::cout << i << ": " << myString << std::endl;
    }
  }

  {
    // uses hashing, similar to unordered_set
    std::unordered_map< std::string, short int>
      myOtherMap = { { "Zero ", 0 },
                     { "One  ", 1 },
                     { "Two  ", 2 },
                     { "Three", 3 },
                     { "Four ", 4 },
                     { "Five ", 5 },
                     { "Six  ", 6 },
                     { "Seven", 7 },
                     { "Eight", 8 },
                     { "Nine ", 9 } };
    // alternative form of initialization

    // iteration as usual (for maps)
    for ( auto it = myOtherMap.begin(); it != myOtherMap.end(); ++it ) {
      std::cout << it->first << " " << it->second << std::endl;
    }

    // note: the unorderred_map is sorted in a non-obvious way
  }

  {
    // multimap can have multiple values per key

    std::multimap<unsigned int, std::string>
      myMap = { { 1, "one" },
                { 1, "eins" },
                { 2, "two" },
                { 2, "zwei" },
                { 3, "three" },
                { 3, "drei" } };
    // no access via operator[]
    for ( auto it = myMap.begin(); it != myMap.end(); ++it ) {
      std::cout << it->first << " " << it->second << std::endl;
    }

    std::cout << "number of occurences of 1 and 4: " << myMap.count ( 1 ) << " " << myMap.count ( 4 ) << std::endl;

    // here's how to iterate over values with the same key
    typedef std::multimap<unsigned int, std::string>::const_iterator citType;
    std::pair< citType, citType >
      range = myMap.equal_range ( 2 );
    for ( citType it = range.first; it != range.second; ++it ) {
      std::cout << it->first << " " << it->second << std::endl;
    }
  }

  return ( EXIT_SUCCESS );
}
